// HCDLL.cpp : Defines the entry point for the DLL application.
//

#include "stdafx.h"
#include <stdio.h>

#ifdef _MANAGED
#pragma managed(push, off)
#endif

extern "C"
{
/*-------------------------------------------------------------------- URegion */
typedef struct URegion {
  long sLon[4],sLat[4];              /* For Uncertainty Regions:              */
                                     /* [0] = closest point (to detecting unit)*/
                                     /* [1] = point clockwise 90 deg from [0] */
                                     /* [2] = farthest point (opposite [0])   */
                                     /* [3] = point opposite [1]              */
                                     /* For Other Regions:                    */
                                     /* [0] = upper left,  [1] = upper right, */
                                     /* [2] = lower right, [3] = lower left   */
  } URegion, * URegionPtr;

typedef unsigned short  ushort;
typedef unsigned char   uchar;
typedef char*			pchar;

const int DLLVERN = 2;
const int DLLVERINTN = 2;
const int SIDES = 3;
const int SLIVER360 = 134217728;

char *LOGFILEOut="HCDLL-Log.txt";
FILE *LOGFILE=NULL;

BOOL APIENTRY DllMain( HMODULE hModule,
                       DWORD  ul_reason_for_call,
                       LPVOID lpReserved
					 )
{
    return TRUE;
}



/*** Init DLL ***/
void _cdecl DLLInit(void)
{
	// here we initialize globals and such for the life of the DLL
	// ALE 20081204 blatantly pulled from Bret's HC code
	if (!LOGFILE)
	{
		LOGFILE = fopen(LOGFILEOut, "w");
		if (LOGFILE)
		{
			fprintf(LOGFILE, "HCDLL Initialized.  Date = %s, Build Time=%s\n", __DATE__, __TIME__);
			fprintf(LOGFILE, "DLLInit\n");
		}
	}
}

/*** Free DLL ***/
void _cdecl DLLFree(void)
{
	// Do any cleanup here, this is the end
	fprintf(LOGFILE, "DLLFree\n");
	// ALE 20081204 blatantly pulled from Bret's HC code
	fflush(LOGFILE);
	fclose(LOGFILE);
}

/*** Return DLL internal version, i.e. revision 5 of my DLL ***/
unsigned short _cdecl DLLVer(void)
{
	fprintf(LOGFILE, "DLLVer,  %d\n", DLLVERN);
	return DLLVERN;
}

/*** Return DLL interface version, i.e. revision 1 of my HCE interface spec ***/
unsigned short _cdecl DLLVerInt(void)
{
	// ALE 20081204 we don't do this because it happens before LOGFILE is opened 
	// fprintf(LOGFILE, "DLLVerInt, %d\n", DLLVERINTN);
	return DLLVERINTN;
}

/*** Return DLL description, 255 characters max ***/
char * _cdecl DLLDesc(void)
{
  fprintf(LOGFILE, "DLLDesc, C++ HC Export DLL by Tony Eischens\n");
  return("C++ HC Export DLL by Tony Eischens");
}


/*** receive lat/lon info ***/
void _cdecl UnitLatLon(uchar uID, pchar gID, double sLat, double sLon)
{
	// Dump data to text file
	fprintf(LOGFILE, "UnitLatLon, Group=%s, Unit=%0.3d, Lat=%8.8f, Lon=%8.8f\n", gID, uID, sLat, sLon);
	// Writeln(txtDataDump, Format('%0.3d, %s, UnitLatLon, %8.8f, %8.8f', [uID, gID, sLat, sLon]));
}

/*** receive visibility info ***/
void _cdecl UnitShowing(uchar uID, pchar gID, uchar showing[SIDES])
{
	// ALE 20081204 as of 2008.064 only showing[0] is used.  The others will be implemented
	//  as time permits so that we know how thoroughly each side has detected each platform
	//  not just how the opposing side has detected it
	fprintf(LOGFILE, "UnitShowing, Group=%s, Unit=%0.3d, %d, %d, %d\n", gID, uID, showing[0], showing[1], showing[2]);
}

/*** receive uncertainty region info ***/
void _cdecl UnitURegion(uchar uID, pchar gID, URegionPtr URegs)
{
	int intC1, intC2;
	char strText[1024], strTmp[256];

	sprintf(strText, "UnitURegion, Group=%s, Unit=%0.3d", gID, uID);
	for (intC1 = 0; intC1 < SIDES-1; intC1++)
	{
		for (intC2 = 0; intC2 < 4; intC2++) // ALE 20081204 four corners in an uncertainty zone
		{
			// Dump data to text file
			// (double)((sLon*360.0)/SLIVER360))
			// (double)(90-(sLat*360.0/SLIVER360))
			sprintf(strTmp, ", %8.8f, %8.8f", (90-(URegs[intC1].sLat[intC2]*360.0/SLIVER360)),
											((URegs[intC1].sLon[intC2]*360.0)/SLIVER360)); 
			strcat(strText, strTmp);
		}
	}
	fprintf(LOGFILE, "%s\n", strText);
}

/*** receive course info ***/
void _cdecl UnitCourse(uchar uID, pchar gID, short course)
{
	fprintf(LOGFILE, "UnitCourse, Group=%s, Unit=%0.3d, Course=%0.3d\n", gID, uID, course);
}

/*** receive speed (kts) info ***/
void _cdecl UnitSpeed(uchar uID, pchar gID, short speed)
{
	fprintf(LOGFILE, "UnitSpeed, Group=%s, Unit=%0.3d, Speed=%0.4d\n", gID, uID, speed);
}

/*** receive altitude band info ***/
void _cdecl UnitAlt(uchar uID, pchar gID, uchar alt)
{
	fprintf(LOGFILE, "UnitAlt, Group=%s, Unit=%0.3d, AltitudeBand=%0.2d\n", gID, uID, alt);
}

/*** receive unit 'color', detected side info ***/
void _cdecl UnitShowingSide(uchar uID, pchar gID, uchar showing[SIDES])
{
	fprintf(LOGFILE, "UnitShowingSide, Group=%s, Unit=%0.3d, %d, %d, %d\n", gID, uID, showing[0], showing[1], showing[2]);
}

/*** receive unit sink info ***/
void _cdecl UnitSink(uchar uID, pchar gID)
{
	fprintf(LOGFILE, "UnitSink, Group=%s, Unit=%0.3d\n", gID, uID);
}

/*** receive gametime ***/
void _cdecl GameTime(long GTime)
{
	fprintf(LOGFILE, "GameTime, %0.6d\n", GTime);
}

/*** receive MessageLog line ***/
void _cdecl MsgLog(pchar MsgLine)
{
	fprintf(LOGFILE, "MsgLog, %s\n", MsgLine);
}

/*** receive Unit assignment to group line ***/
void _cdecl UnitToGroup(uchar uID, pchar gID, pchar gIDold)
{
	fprintf(LOGFILE, "UnitToGroup, Group=%s, Unit=%0.3d, OldGroup=%s\n", gID, uID, gIDold);
}

/*** receive new unit info ***/
void _cdecl UnitNew(uchar uID, pchar gID, char utype, uchar side, ushort annexID, pchar name)
{
	fprintf(LOGFILE, "UnitNew, Group=%s, Unit=%0.3d, Type=%0.3d, Side=%d, DiskID=%0.5d, Name=%s\n", gID, uID, utype, side, annexID, name);
}

/*** receive new unit free info, after a unit sinks eventually it is freed, planes free right away, etc. ***/
void _cdecl UnitFree(uchar uID, pchar gID)
{
	fprintf(LOGFILE, "UnitFree, Group=%s, Unit=%0.3d\n", gID, uID);
}

/*** receive new scenario/save info ***/
void _cdecl ScenLoad(pchar strHarpPath, short intScenNum, pchar strScenFPath, char intBSNum, pchar strBSFName, uchar intFriendlySide)
{
	fprintf(LOGFILE, "ScenLoad, %s, %d, %s, %d, %s, %d\n", strHarpPath, intScenNum, strScenFPath, intBSNum, strBSFName, intFriendlySide);
}

} // extern "C"

#ifdef _MANAGED
#pragma managed(pop)
#endif

